# -*- coding: utf-8 -*-

'''
tooner.invasions module
~~~~~~~~~~~~~~~~~~~~~~~~~~~

Functions and classes related to communicating with the Toontown
Rewritten API and tracking invasions.
'''

import requests


class InvasionTracker:
    '''Pulls invasion information from the Toontown Rewritten API.
    
    Uses the given login information to communicate with the
    Toontown Rewritten API and get information about invasions that are 
    currently happening.

    This class simply communicates with the Toontown Rewritten API to 
    get the latest invasion information. It also does a very basic 
    cleaning of the API's response.
    
    Properties:
        invasions:
            Returns a dictionary of current Toontown invasions, where 
            the key is the district that is being invaded and the value 
            is the cog that is invading that district.
        cogs:
            Returns a list of cogs that are currently invading 
            Toontown.
        districts:
            Returns a list of districts that are currently being 
            invaded.
    '''

    def __init__(self):
        '''Please see help(InvasionTracker) for more info.'''

        self.api_url = 'https://www.toontownrewritten.com/api/invasions'
    
    @property
    def invasions(self):
        '''A simple wrapper around the get_invasions method.'''

        return self.get_invasions()
    
    @property
    def cogs(self):
        '''Returns a list of currently-invading cogs.'''

        cogs = [invasion['type'] for invasion in self._invasions.values()]
        cogs = [cog.replace('\x03', '') for cog in cogs]
        return cogs
    
    @property
    def districts(self):
        '''Returns a list of currently-invaded districts.'''

        districts = list(self._invasions.keys())
        return districts

    def get_invasions(self):
        '''Returns information about currently invading cogs.'''

        return {k: self._clean(v['type']) for k, v in self._invasions.items()}

    def _make_request(self):
        '''Makes a get request to the Toontown Rewritten API.
        
        Returns the raw json response from the API, which includes 
        information about Toontown's current invasions.'''

        # Make a get request to the Toontown Rewritten API
        request = requests.get(self.api_url)
        # Return the json data from the response
        return request.json()
    
    def _clean(self, string):
        '''Removes undesired information/characters from a string.
        
        For some cogs, some byte artifacts (such as \x03) are still 
        included in the string. These aren't typically desired, so they 
        are replaced with an empty string.

        Args:
            string (str):
                The string to clean.
        '''

        return string.replace('\x03', '')

    @property
    def _invasions(self):
        '''Returns relevant information from the API's json response.'''

        return self._make_request()['invasions']



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tooner']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.23.0,<3.0.0']

setup_kwargs = {
    'name': 'tooner',
    'version': '1.0.5',
    'description': 'An easier way to manage and launch sessions for Toontown Rewritten.',
    'long_description': '<div align="center">\n\n  <img src="https://github.com/jakebrehm/tooner/blob/master/img/logo.png" alt="Tooner Logo"/>\n\n  <br>\n  <br>\n\n  <h1>An easier way to launch sessions and track invasions for Toontown Rewritten.</h1>\n\n  <br>\n\n  <img src="https://img.shields.io/github/last-commit/jakebrehm/tooner?style=for-the-badge&color=yellow" alt="Last Commit"></img>\n  <img src="https://img.shields.io/github/commit-activity/w/jakebrehm/tooner?style=for-the-badge&color=yellow" alt="Commit Activity"></img>\n  <img src="https://img.shields.io/github/license/jakebrehm/tooner?style=for-the-badge&color=yellow" alt="MIT License"></img>\n  <br>\n  <img src="https://img.shields.io/badge/Made%20With-Python%203.7-violet.svg?style=for-the-badge&logo=Python" alt="Made with Python 3.7"></img>\n\n</div>\n\n<br>\n\n## What does it do?\n\nCurrently, **tooner** allows you to communicate with Toontown Rewritten\'s login API in order to log in and start a session with very few lines of code.\n\n```python\nlauncher = tooner.ToontownLauncher(directory="...")\nlauncher.play(username="username", password="password")\n```\n\nIf you\'re crazy, you can even combine these lines into one!\n\nAll you have to do is supply the directory of your Toontown Rewritten installation (where the TTREngine is stored) and your login information. On Windows, check your program files directories. On MacOS, this is in your Application Support directory. Eventually, I\'d like to make this library automatically find the installation.\n\nThe best part is that you can do this to **play multiple toons at once**.\n\nYou are also able to **get the latest invasion information**.\n\n```python\ntracker = tooner.InvasionTracker()\ninvasions = tracker.invasions\n```\n\nUnfortunately, *latest* does not mean *freshest* in this case, as fresh information is pushed to the Toontown Rewritten Invasion API noticeably infrequently.\n\n## Why does this exist?\n\nSince I normally play on MacOS, there is no way for me to open multiple sessions of the Toontown Rewritten launcher without doing it from the terminal; this was really annoying to do every time I wanted to multitoon (which is a lot), so I set out to make this easier.\n\nUltimately, I was successful in making this functionality work the three major operating systems: Windows, MacOS, and, I assume, on Linux (I haven\'t been able to test this).\n\n## How do I get it?\n\nIt\'s easiest to simply install the package via pip using the following command:\n\n```\npip install tooner\n```\n\nOtherwise, you can close this repository using the command\n\n```\ngit clone https://github.com/jakebrehm/tooner.git\n```\n\nand then you can do whatever you want with it!\n\n## Future improvements\n\nMy two main concerns and top priorities for this package are the following:\n\n- **Automatically finding the directory,** or at least try. I imagine users should still be able to supply their own directory if they know the program is installed in a non-default location. This would further (and possibly significantly) simplify the usage of the package, and would allow for a one-line launcher.\n- **Adding support for ToonGuard**. The only problem is writing it in such a way that makes sense while keeping in mind *tooner*\'s two-line launcher paradigm, as it would require the user to enter a code *after* having run the script.\n- **Make invasion functions**. Instead of having to make a class and keep track of it, some basic functions to encapsulate the class would be useful and easy to implement.\n\n## Projects using **tooner**\n\nThe following projects are using **tooner**:\n1. [MultiTooner](https://github.com/jakebrehm/multitooner) by [Jake Brehm](https://github.com/jakebrehm)\n\n## Authors\n\n- **Jake Brehm** - *Initial Work* - [Email](mailto:mail@jakebrehm.com) | [Github](http://github.com/jakebrehm) | [LinkedIn](http://linkedin.com/in/jacobbrehm)',
    'author': 'Jake Brehm',
    'author_email': 'mail@jakebrehm.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-

'''
tooner.launcher module
~~~~~~~~~~~~~~~~~~~~~~~~~~~

Functions and classes related to communicating with the Toontown
Rewritten API, logging in, and opening the launcher.
'''

import os
import platform
import subprocess
import time

import requests


class ToontownLauncher:
    '''Uses the given login information to communicate with the
    Toontown Rewritten API in an attempt to log in and launch the game.
    
    Args:
        directory (str):
            The directory of your Toontown Rewritten installation.
    
    Kwargs:
        attempts (int) [5]:
            The maximum number of attempts to connect before giving up.
            Accepts a None value to try infinitely, but this is not
            recommended.
        debug (bool) [True]:
            Whether to write information to the console detailing
            communication with the API.
    '''

    def __init__(self, directory, attempts=5, debug=False):
        '''Please see help(ToontownLauncher) for more info.'''

        self.directory = directory
        self.api_url = 'https://www.toontownrewritten.com/api/login?format=json'

        self._attempts = 0
        self.maximum_attempts = attempts

        self._debug = debug
        self._stdout = None if debug else subprocess.DEVNULL

    def play(self, **data):
        '''A more intuitively-named wrapper for the _connect method.'''

        return self._connect(**data)

    def _message(self, message):
        '''Print a message to the console.'''

        if self._debug:
            print(message)

    def _connect(self, **data):
        '''Communicate with the Toontown Rewritten API in an attempt to
        connect and log in.
        
        Kwargs:
            username (str):
                The username of the Toontown Rewritten account.
            password (str):
                The password of the Toontown Rewritten account.
            appToken (str):
                The ToonGuard code generated by an authenticator app.
                Only required if ToonGuard is activated for the
                Toontown Rewritten account.

        Returns:
            True: launch was successful.
            False: launch was unsuccessful.
            None: ToonGuard code is required.
        '''

        # Increment attempts counter and stop if the maximum has been reached
        self._attempts += 1
        if self.maximum_attempts and self._attempts > self.maximum_attempts:
            self._message(
                'The maximum number of attempts has been reached. '
                'Please try again.'
            )
            return False

        # If the connection request 
        app_token = data.pop('appToken', None)

        # Post a request and determine its success
        response = self._make_request(data)
        success = response.get('success', 'false')

        # If the connection was successful, proceed to launch the game
        if success == 'true':
            cookie = response.get('cookie', 'CookieNotFound')
            game_server = response.get('gameserver', 'ServerNotFound')
            self._message('Successful granted access.')
            self._launch_game(cookie, game_server)
            return True
        # If the success was partial, a ToonGuard code is required to log in
        elif success == 'partial':
            authorization_token = response.get('responseToken', None)
            if not app_token or not authorization_token:
                self._message('ToonGuard code required.')
                return None
            time.sleep(3)
            self._connect(appToken=app_token, authToken=authorization_token)
        # If success was delayed, the user was placed in a queue
        elif success == 'delayed':
            queue_token = response.get('queueToken', None)
            if not queue_token:
                self._message('Something went wrong. Please try again.')
                return False
            position = response.get('position')
            self._message(f'You are currently queued in position {position}.')
            time.sleep(3)
            self._connect(queueToken=queue_token)
        # Otherwise, the connection has failed
        else:
            self._message(
                'Connection failed. Something went wrong, but maybe it was '
                'just your username and password.'
            )
            return False

    def _make_request(self, data):
        '''Communicates with the Toontown Rewritten API to post a
        request.
        
        Args:
            data (dict):
                The data to send with the request.    
        '''

        # Post a request to the Toontown Rewritten API
        post = requests.post(
            self.api_url,
            data=data,
            headers={'Content-type': 'application/x-www-form-urlencoded'},
        )
        # Return the json data from the response
        return post.json()

    def _launch_game(self, play_cookie, game_server):
        '''Use the given response credentials to open the luancher and
        log into the game.
        
        Args:
            play_cookie (str):
                The cookie key used to log in. Received from the
                Toontown Rewritten API.
            game_server (str):
                The gameserver key used to log in. Received from the
                Toontown Rewritten API.
        '''

        # Determine the user's operating system
        operating_system = platform.system()
        # Change to the Toontown Rewritten directory
        os.chdir(self.directory)

        # Set the enironmental variables including login information
        os.environ['TTR_PLAYCOOKIE'] = play_cookie
        os.environ['TTR_GAMESERVER'] = game_server

        # Start the Toontown Rewritten process
        if operating_system == 'Windows':
            subprocess.Popen(
                args="TTREngine.exe",
                stdout=self._stdout,
                creationflags=0x08000000,
            )
        elif operating_system == 'Linux':
            # TODO: Linux features are currently untested.
            subprocess.Popen(args="./TTREngine", stdout=self._stdout)
        elif operating_system == 'Darwin':
            subprocess.Popen(args='./Toontown Rewritten', stdout=self._stdout)
        else:
            self._message('Your operating system is not currently supported.')
    
        # Let the user know the connection was successful
        self._message('Successfully connected.')
